
package MT::Plugin::RightFields;
use strict;

use vars qw( $VERSION );
$VERSION = '1.2b1';

use base qw( MT::Plugin );

require MT::Plugin;
require MT;
require MT::Entry;

my $rightfields;
$rightfields = MT::Plugin::RightFields->new({
	'name' => 'RightFields',
	'author_name' => 'Kevin Shay',
	'author_link' => 'http://www.staggernation.com/',
	'description' => 'Tailor the entry fields for each weblog to meet your needs.',
	'version' => $VERSION,
	'config_link' => 'mt-rightfields.cgi',
	'callbacks' => {
		'CMSPostSave.entry' => \&CMSPostSave_entry,
		'CMSPreSave.entry' => \&CMSPreSave_entry,
		'MT::Entry::pre_remove' => \&entry_pre_remove,
		
		'MT::App::CMS::AppTemplateSource.edit_entry' => \&edit_entry_template,
		'MT::App::CMS::AppTemplateParam.edit_entry', => \&edit_entry_param,
		'MT::App::CMS::AppTemplateParam.preview_entry' => \&preview_entry_param,
		'MT::App::CMS::AppTemplateParam.list_entry' => \&list_entry_param,
		'MT::App::CMS::AppTemplateSource.entry_table' => \&entry_table_template
	},
	'template_tags' => {
		'ExtraFieldValue' => \&hdlr_extra_field_value,
		'FieldValue' => \&hdlr_field_value,
		'FieldKey' => \&hdlr_field_key,
		'FieldLabel' => \&hdlr_field_label,
		'ExtraFieldLabel' => \&hdlr_field_label,
		'FileURLPath' => \&hdlr_file_url_path,
	},
	'container_tags' => {
		'ExtraFields' => \&hdlr_extra_fields,
		'ExtraFieldsLoop' => \&hdlr_extra_fields_loop,
		'LinkedEntry' => \&hdlr_linked_entry,
		'LinkingEntries' => \&hdlr_linking_entries,
		'FormatDate' => \&hdlr_format_date
	},
	'conditional_tags' => {
		'IfExtraFields' => \&hdlr_if_extra_fields,
		'IfExtraField' => \&hdlr_if_extra_field,
		'IfLinkingEntries' => \&hdlr_if_linking_entries
	}
});
MT->add_plugin($rightfields);


	# in order to implement custom template tags without defining them all
	# every time the plugin is loaded, but only when they're needed for a
	# template context, we hook into MT::Builder
require MT::Builder;
require MT::Template::Context;
require MT::Template::ContextHandlers;
	# store pointers to the original subroutines
my $mt_builder_build = \&MT::Builder::build;
my $mt_edit_object = \&MT::App::CMS::edit_object;
my $mt_search_hit = \&MT::App::Search::_search_hit;
my $mt_hdlr_entries = \&MT::Template::Context::_hdlr_entries;
my $mt_if_nonempty = \&MT::Template::Context::_hdlr_if_nonempty;
	# install our new subroutines
{
	local $SIG{__WARN__} = sub {  }; 
	*MT::Builder::build = \&builder_build;
	*MT::App::CMS::edit_object = \&_edit_object;	
	*MT::App::Search::_search_hit = \&search_hit;	
	*MT::Template::Context::_hdlr_entries = \&hdlr_entries;	
	*MT::Template::Context::_hdlr_if_nonempty = \&hdlr_if_nonempty;	
}

my $custom_tags_loaded = {};

sub instance { $rightfields }

sub builder_build {
	(my $blog_id = $_[1]->stash('blog_id')) || return &{$mt_builder_build}(@_);
	$custom_tags_loaded->{$blog_id} && return &{$mt_builder_build}(@_);
	require RightFields;
	my $tags = RightFields::blog_data('tags', $blog_id) || [];
	if (@$tags) {
		for my $tag (@$tags) {
			$MT::Template::Context::Handlers{$tag->{'tag'}} = 
				sub{&hdlr_custom_tag(@_, $tag->{'field'})};
		}
	}
	$custom_tags_loaded->{$blog_id} = 1;
	&{$mt_builder_build}(@_);
}

sub _edit_object {
	my ($app, $param) = @_;
	my $q = $app->{'query'};
		# we only care about re-editing an entry from Preview
	if ($q->param('reedit') && ($q->param('_type') eq 'entry')) {
		for my $key ($q->param) {
			if ($key =~ /^rf_/) {
				$param->{$key} = $q->param($key);
			}
		}
	}
	return &{$mt_edit_object}($app, $param);
}

sub hdlr_if_nonempty {
	my ($ctx, $args, $cond) = @_;
	$args->{'tag'} || &{$mt_if_nonempty}(@_);
	(my $blog_id = $ctx->stash('blog_id')) || return &{$mt_if_nonempty}(@_);
	require RightFields;
	my $tags = RightFields::blog_data('tags', $blog_id) || [];
	my %tag_names = map { $_->{'tag'} => $_->{'field'} } @$tags;
	if ($tag_names{$args->{'tag'}}) {
		my $value = RightFields::hdlr_custom_tag($ctx, $args, $cond, 
		$tag_names{$args->{'tag'}});
		if (defined($value) && $value ne '') { # want to include "0" here
			return MT::Template::Context::_hdlr_pass_tokens($ctx, $args, $cond);
		} else {
			return MT::Template::Context::_hdlr_pass_tokens_else($ctx, undef, $cond);
		}
	}
	return &{$mt_if_nonempty}(@_);
}

sub search_hit {
	require RightFields;
	return RightFields::search_hit($mt_search_hit, @_);
}

sub hdlr_entries {
	require RightFields;
	return RightFields::hdlr_entries($mt_hdlr_entries, @_);
}

sub convert_data {
	require RightFields;
	return RightFields::convert_data(@_);
}

sub CMSPreSave_entry {
	require RightFields;
	return RightFields::CMSPreSave_entry(@_);
}

sub CMSPostSave_entry {
	require RightFields;
	return RightFields::CMSPostSave_entry(@_);
}

sub entry_pre_remove {
	require RightFields;
	return RightFields::entry_pre_remove(@_);
}

sub edit_entry_template {
	require RightFields;
	return RightFields::edit_entry_template($rightfields, @_);
}


sub edit_entry_param {
	require RightFields;
	return RightFields::edit_entry_param(@_);
}

sub preview_entry_param {
	require RightFields;
	return RightFields::preview_entry_param(@_);
}

sub list_entry_param {
	require RightFields;
	return RightFields::list_entry_param(@_);
}

sub entry_table_template {
	require RightFields;
	return RightFields::entry_table_template(@_);
}

	# TEMPLATE TAG HANDLERS

sub hdlr_custom_tag {
	require RightFields;
	return RightFields::hdlr_custom_tag(@_);
}

sub hdlr_linked_entry {
	require RightFields;
	return RightFields::hdlr_linked_entry(@_);
}

sub hdlr_linking_entries {
	require RightFields;
	return RightFields::hdlr_linking_entries(@_);
}

sub hdlr_if_linking_entries {
	require RightFields;
	return RightFields::hdlr_if_linking_entries(@_);
}

sub hdlr_extra_fields {
	require RightFields;
	return RightFields::hdlr_extra_fields(@_);
}

sub hdlr_if_extra_fields {
	require RightFields;
	return RightFields::hdlr_if_extra_fields(@_);
}

sub hdlr_if_extra_field {
	require RightFields;
	return RightFields::hdlr_if_extra_field(@_);
}

sub hdlr_extra_fields_loop {
	require RightFields;
	return RightFields::hdlr_extra_fields_loop(@_);
}

sub hdlr_extra_field_value {
	require RightFields;
	return RightFields::hdlr_extra_field_value(@_);
}

sub hdlr_field_value {
	require RightFields;
	return RightFields::hdlr_field_value(@_);
}

sub hdlr_field_key {
	require RightFields;
	return RightFields::hdlr_field_key(@_);
}

sub hdlr_field_label {
	require RightFields;
	return RightFields::hdlr_field_label(@_);
}

sub hdlr_file_url_path {
	require RightFields;
	return RightFields::hdlr_file_url_path(@_);
}

sub hdlr_format_date {
	require RightFields;
	return RightFields::hdlr_format_date(@_);
}

sub hdlr_show_key {
	require RightFields;
	return RightFields::hdlr_show_key(@_);
}

sub hdlr_show_value {
	require RightFields;
	return RightFields::hdlr_show_value(@_);
}

1;