
package RightFields;
use strict;
use Data::Dumper;

use Exporter;
@RightFields::ISA = qw( Exporter );
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( load_plugindata save_plugindata config_key choice_tmpl
	mt_fields mt_field_order fld_name new_obj load_rf_settings save_rf_settings
	current_ts );

use MT::Util qw( remove_html offset_time_list );

my $plugin_key = 'rightfields';
my $blog_data = { 'standard' => {}, 'extra' => {}, 'tags' => {} };
my $default_data = {};
my $row_height = 18; # approx. height of a row in a textarea, for resizing

my $mt_fields = {
	'title' => 'Title',
	'text' => 'Entry Body',
	'text_more' => 'Extended Entry',
	'excerpt' => 'Excerpt',
	'keywords' => 'Keywords'
};

my $mt_field_order = {
	'title' => '1',
	'text' => '2',
	'text_more' => '3',
	'excerpt' => '4',
	'keywords' => '5'
};

sub mt_fields { $mt_fields }
sub mt_field_order { $mt_field_order }

sub load_plugindata {
	my ($key) = @_;
	require MT::PluginData;
	my $data = MT::PluginData->load({
		plugin => $plugin_key, key => $key
	});
	return 0 unless $data;
	return $data->data;
}

sub save_plugindata {
	my ($key, $data) = @_;
	require MT::PluginData;
	my $plugindata = MT::PluginData->load({
		'plugin' => $plugin_key, 'key' => $key
	});
	if (!$plugindata) {
		$plugindata = MT::PluginData->new;
		$plugindata->plugin($plugin_key);
		$plugindata->key($key);
	}
	$plugindata->data($data);
	$plugindata->save || return 0;
}

sub config_key {
	my ($blog_id, $type) = @_;
	if ($blog_id == -1) {
		return "default_$type";
	} else {
		return "blog_${blog_id}_cfg_$type";
	}
}

sub load_rf_settings {
	my ($q, $key) = @_;
	my $data_key = config_key($q->param('blog_id'), $key);
	return load_plugindata($data_key);
}

sub save_rf_settings {
	my ($q, $key, $data) = @_;
	my $data_key = config_key($q->param('blog_id'), $key);
	return save_plugindata($data_key, $data);
}

sub new_obj {
	my ($blog_config, $entry_id) = @_;
	my $obj;
	my $class = ($blog_config->{'datasource'} eq '_pseudo')
		? 'RightFieldsPseudo' : 'RightFieldsObject';
	eval "require $class";
	$class->set_properties(obj_properties($blog_config));
	if ($entry_id) {
		$obj = $class->load(
			($class eq 'RightFieldsPseudo') ? { 'key' => $entry_id } : $entry_id
		);
	}
	if (!$obj) {
		$obj = $class->new;
	}
	return $obj;
}

sub obj_properties {
	my ($blog_config) = @_;
	my @e_id = ($blog_config->{'datasource'} eq '_pseudo') ? ('entry_id') : ();
	return {
		'columns' => [ 'id', @e_id, keys %{$blog_config->{'cols'}} ],
		'datasource', $blog_config->{'datasource'}
	};
}

sub fld_name {
# prefix column names consistently
	my ($key) = @_;
	return "rf_$key";
}

sub blog_data {
# return cached or load data of a specified type and blog ID
	my ($type, $blog_id) = @_;
	if (!$blog_data->{$type}->{$blog_id} 
			&& !ref($blog_data->{$type}->{$blog_id})) {
		my $cfg_key = config_key($blog_id, $type);
		$blog_data->{$type}->{$blog_id} = load_plugindata($cfg_key);
	}
		# no data for this blog found, try defaults
	if (!$blog_data->{$type}->{$blog_id}) {
		if (!$default_data->{$type}) {
			my $cfg_key = config_key(-1, $type);
			$default_data->{$type} = load_plugindata($cfg_key);
		}
		return $default_data->{$type};
	}
	return $blog_data->{$type}->{$blog_id};
}

	# CALLBACK SUBROUTINES

sub CMSPreSave_entry {
    my ($cb, $app, $entry) = @_;
	my $q = $app->{'query'};

	if (!$entry->id) {
		my $defaults = blog_data('defaults', $q->param('blog_id')) || {};
		my $fields = mt_fields();
		warn Dumper($defaults);
		for my $field (keys %$fields) {
				# check for guide string in a field that wasn't entered
			if ($defaults->{"std_$field"} && $defaults->{"std_${field}_is_guide"}
					&& $entry->$field
					&& ($entry->$field eq $defaults->{"std_$field"})) {
				$entry->$field('');
			}
		}
	}
	my $cfg = blog_data('standard', $q->param('blog_id')) || {};
	return 1 unless ($cfg && %$cfg && rf_enabled($cfg, $q));
		# deal with special field types
	for my $field (keys %$cfg) {
		next unless ref($cfg->{$field});
		if ($cfg->{$field}->{'customize'}) {
			if ($cfg->{$field}->{'type'} eq 'checkbox') {
				$entry->$field(0) unless defined($q->param($field));
			} elsif ($cfg->{$field}->{'type'} eq 'date') {
				$entry->$field(params_to_ts($q, $field));
			} elsif ($cfg->{$field}->{'type'} eq 'file') {
				my $upload_field = $field . '_upload';
				next unless ($q->param($upload_field));
				$entry->$field(handle_upload($entry, $q, $cfg->{$field}, $field, 
					$field, $upload_field));
			}
		}
	}
	return 1;
}

sub CMSPostSave_entry {
	my ($cb, $app, $entry) = @_;
	my $q = $app->{'query'};
	return 0 if ($q->param('return_args') && 
		($q->param('return_args') =~ /is_power_edit=1/));
	my $cfg = blog_data('extra', $q->param('blog_id')) || {};
	return 1 unless ($cfg && %$cfg && rf_enabled($cfg, $q));
	my $defaults = blog_data('defaults', $q->param('blog_id')) || {};
		# deal with special field types
	for my $col (keys %{$cfg->{'cols'}}) {
		my $field = fld_name($col);
		if (!$q->param('id')) {
			# check for guide string in a field that wasn't entered
			if ($defaults->{$col} && $defaults->{$col . '_is_guide'}
					&& $q->param($field)
					&& ($q->param($field) eq $defaults->{$col})) {
				$q->param($field, '');
			}
		}
		if ($cfg->{'cols'}->{$col}->{'type'} eq 'checkbox') {
			if (!defined($q->param($field))) {
				$q->param($field, 0);
			}
		} elsif ($cfg->{'cols'}->{$col}->{'type'} eq 'date') {
			$q->param($field, params_to_ts($q, $field));
		} elsif ($cfg->{'cols'}->{$col}->{'type'} eq 'file') {
			my $upload_field = $field . '_upload';
			next unless ($q->param($upload_field));
			my $filename = handle_upload($entry, $q,
				$cfg->{'cols'}->{$col}, $field, $col, $upload_field);
				# need to stringify the filename in some cases
			$q->param($field, "$filename");
		}
	}
	my $obj = new_obj($cfg, $entry->id);
	if ($cfg->{'datasource'} eq '_pseudo') {
		$obj->entry_id($entry->id);
	} else {
		$obj->id($entry->id);
	}
	my %vals = map { $_ => scalar $q->param(fld_name($_)) }
		keys %{$cfg->{'cols'}};
		# don't create a new row if they didn't enter any values
	return 0 if ((length(join('', values %vals)) == 0) && (!$obj->exists));

	$obj->set_values(\%vals);
	$obj->save || die $obj->errstr;
	return 1;
}

sub params_to_ts {
	my ($q, $field) = @_;
	my %date = ();
	for my $key (qw( y m d h min s )) {
		$date{$key} = $q->param("${field}_$key") || 0;
	}
	if ($q->param("${field}_ampm") && ($q->param("${field}_ampm") eq 'pm')) {
		$date{'h'} += 12;
		if ($date{'h'} == 24) {
			$date{'h'} = 12;
		}
	}
	my $ts = sprintf("%04d%02d%02d%02d%02d%02d", @date{qw( y m d h min s )});
	return ($ts ne '00000000000000') ? $ts : '';
}

sub handle_upload {
	my ($entry, $q, $cfg, $field, $col, $upload_field) = @_;
		# if they entered a value in the actual field, use that;
		# if not, use name of uploaded file
	my $upload_file;
	if ($q->param($field)) {
		$upload_file = $q->param($field);
	} else {
		$upload_file = $q->param($upload_field);
			# IE/Win sends full path as filename
		if ($upload_file =~ m#[/\\]#) {
			$upload_file =~ s#.*[/\\]([^/\\]+)$#$1#;
		}
		if ($cfg->{'filenames'} ne 'keep') {
			$upload_file =~ s/\.(\w+)$//;
			my $ext = $1;
			if (!$ext) {
				(undef, $ext) = split(m#/#, 
					$q->uploadInfo($q->param($upload_field))->{'Content-Type'});
				$ext = 'jpg' if ($ext eq 'jpeg');
				$ext = 'txt' if ($ext eq 'text');
			}
			if ($cfg->{'filenames'} eq 'dirify') {
				require MT::Util;
				$upload_file = MT::Util::dirify($upload_file);
			} elsif ($cfg->{'filenames'} eq 'id') {
				$upload_file = $entry->id;
			} elsif ($cfg->{'filenames'} eq 'basename') {
				$upload_file = $entry->basename;
			}
			$upload_file .= "_${col}.$ext";
		}
	}
	
	return save_upload($q->upload($upload_field),
		$cfg->{'upload_path'}, $upload_file, $cfg->{'overwrite'});
}

sub save_upload {
	my ($fh, $path, $file, $overwrite) = @_;
	$path =~ s#/$##;
		# if no path, don't add a slash before filename
		# (i.e., filename itself is absolute path)
	$path = "$path/" if $path;
	my $filename = "$path$file";
	my $newfile;
		# generate unique filename (add number if it exists)
	unless ($overwrite) {
		my $i = 0;
		while (-e $filename) {
			$i++;
			$newfile = $file;
			$newfile =~ s/\.(\w+)$/_$i.$1/;
			$filename = "$path$newfile";
		}
	}
	seek($fh, 0, 0);
	open(UPLOAD, ">$filename");
	binmode(UPLOAD);
	while (<$fh>) {
		print UPLOAD;
	}
	close UPLOAD;
	return $newfile ? $newfile : $file;
}

sub entry_pre_remove {
	my ($cb, $entry) = @_;
	my $blog_id = $entry->blog_id;
	my $cfg = blog_data('extra', $blog_id) || {};
	if ($cfg && %$cfg) {
		my $obj = new_obj($cfg, $entry->id);
		if ($obj && $obj->id) {
			$obj->remove;
		}
	}
}

sub guide_js {
	my ($field, $value) = @_;
	$field = uc($field);
	$value ||= "<TMPL_VAR NAME=$field>";
	return <<"EOH";
 onfocus="this.value='';return false;" onblur="if(!this.value) this.value='$value';return false;"
EOH
}

sub edit_entry_template {
	my ($rightfields, $cb, $app, $template) = @_;
	my $q = $app->{'query'};
	my $old = q{name="entry_form"};
	my $new = q{name="entry_form"<TMPL_IF NAME=RF_HAS_FILE> enctype="multipart/form-data"</TMPL_IF>};
	$$template =~ s/$old/$new/;
	$old = '</script>';
	$new = <<HTML;
</script>
<TMPL_IF NAME=RF_HAS_DATE>
<script type="text/javascript">
function rf_set_menu (menu, value) {
	for (var i = 0; i < menu.options.length; i++) {
		if (menu.options[i].value == value) {
			menu.selectedIndex = i;
			break;
		}
	}
}
function rf_cal_set (field, y, m, d) {
     rf_set_menu(getByID(field + '_y'), y);
     rf_set_menu(getByID(field + '_m'), m);
     rf_set_menu(getByID(field + '_d'), d);
}
</script>
</TMPL_IF>
HTML
	$$template =~ s/$old/$new/;
	my $blog_id = $q->param('blog_id');
	if ($app->{'author'}->is_superuser() ||
		($app->{'perms'} && $app->{'perms'}->can_administer_blog)) {
		$old = 'page."></a></p>';
		my $rf_link = "$rightfields->{'envelope'}/$rightfields->{'config_link'}?blog_id=$blog_id";
		$new = $old . <<TMPL;
<p>RightFields settings: <a href="$rf_link&__mode=cfg_standard">Standard Fields</a> | <a href="$rf_link&__mode=cfg_extra">Extra Fields</a> | <a href="$rf_link&__mode=cfg_defaults">Default Values</a></p>
TMPL
		$$template =~ s/$old/$new/;
	}

	my $std = blog_data('standard', $blog_id);
	if (rf_enabled($std, $q)) {
		template_standard($std, $template);
	}
	my $extra = blog_data('extra', $blog_id);
	if (rf_enabled($extra, $q)) {
		template_extra($extra, $template);
	}
	my $defaults = blog_data('defaults', $blog_id);
		# this will take care of standard, extra is handled in param
	if ($defaults) {
		for my $key (grep { /^std_.+_is_guide$/ } keys %$defaults) {
			next unless ($defaults->{$key} eq 'on');
			$key =~ s/_is_guide$//;
			$key =~ s/^std_//;
			next if ($std->{$key}->{'customize'}
				&& ($std->{$key}->{'type'} !~ /^text/));
			my $guide_js = guide_js($key);
			$$template =~ s#name="$key"#name="$key"<TMPL_UNLESS NAME=ID>$guide_js</TMPL_UNLESS>#;
		}
	}
}

sub template_standard {
	my ($cfg, $template) = @_;
	
	my $fields = mt_fields();
	my $fields_html;
	for my $field (keys %$fields) {
		next unless ($cfg && %$cfg && $cfg->{$field}->{'customize'});
		my $ucfield = uc($field);
		my $old_label = qq{<MT_TRANS phrase="$fields->{$field}">};
		my $new_label = qq{<MT_TRANS phrase="$cfg->{$field}->{'label'}">};
		$fields_html ||= fields_html();
		my $old = $fields_html->{$field};
		my $new = $old;
		if ($field =~ /^text/) {
			if (($cfg->{$field}->{'type'} ne 'textarea')
					|| (defined($cfg->{$field}->{'show_buttons'}) &&
					($cfg->{$field}->{'show_buttons'} == 0))) {
				my $helpcode = ($field eq 'text') ? 'body' : 'extended_entry';
				my $buttons = qq{'$helpcode')" class="help">?</a>
</div>
<div class="field-wrapper">
<div class="field-buttons">
<div class="field-buttons-formatting">
<script type="text/javascript">
if (canFormat)};
				my $newbuttons = $buttons;
				$newbuttons =~ s/canFormat/0/; # heh
				$newbuttons =~ s/div class="[^"]+"/div class="xxx"/g;
				$buttons = quotemeta($buttons);
				$$template =~ s/$buttons/$newbuttons/;
				my $resizing_buttons = field_buttons_resizing();
				$resizing_buttons =~ s/__FIELD__/$field/g;
				$resizing_buttons = quotemeta($resizing_buttons);
				$$template =~ s/$resizing_buttons//s;
			}
		} elsif (($cfg->{$field}->{'type'} eq 'textarea')
				&& (defined($cfg->{$field}->{'show_buttons'}) &&
				($cfg->{$field}->{'show_buttons'} == 1))) {
			my $buttons = field_buttons_formatting() . field_buttons_resizing();
			$buttons =~ s/__FIELD__/$field/g;
			my $height = $cfg->{$field}->{'rows'} * $row_height;
			$new = <<"HTML";
<div class="field-wrapper">
<div class="field-buttons">
$buttons
</div>
$new
<input type="hidden" name="${field}_height" id="${field}_height" value="$height" />
</div>
HTML
		}
		if (($field eq 'title') && ($cfg->{$field}->{'type'} eq 'textarea')) {
				# need to change from input to textarea
			$new =~ s#<input#<textarea#;
			$new =~ s# value="<TMPL_VAR NAME=TITLE ESCAPE=HTML>"##;
			$new =~ s# />#><TMPL_VAR NAME=TITLE ESCAPE=HTML></textarea>#;
			my $style = qq{style="width:$cfg->{$field}->{'width'}px;"};
			my $rows = qq{rows="$cfg->{$field}->{'rows'}"};
			$new =~ s#(name="title")#$1 $style $rows#;
		} elsif (($field eq 'title') && ($cfg->{$field}->{'type'} eq 'text')) {
			my $style = qq{style="width:$cfg->{$field}->{'width'}px;"};
			$new =~ s#(name="title")#$1 $style#;
			$new =~ s#255#$cfg->{$field}->{'length'}#;
		} elsif ($cfg->{$field}->{'type'} eq 'text') {
				# need to change from textarea to input
			my $style = qq{style="width:$cfg->{$field}->{'width'}px;"};
			$new =~ s#"textarea-wrapper"#"textarea-wrapper" $style#;
			$new =~ s#<textarea#<input $style#;
			my $tmpl_var = "<TMPL_VAR NAME=$ucfield ESCAPE=HTML>";
			$new =~ s/ style="height: [^"]+"//;
			my $length = $cfg->{$field}->{'length'}
				? qq{maxlength="$cfg->{$field}->{'length'}"}: '';
			$new =~ s#>$tmpl_var</textarea># $length value="$tmpl_var" />#;
		} elsif ($cfg->{$field}->{'type'} eq 'textarea') {
			$new =~ s/"textarea-wrapper"/"textarea-wrapper" style="width:$cfg->{$field}->{'width'}px;"/;
			$new =~ s#rows="[^"]+"#rows="$cfg->{$field}->{'rows'}"#;
			$new =~ s/"height: [^"]+"/"width:$cfg->{$field}->{'width'}px;"/;
			my $height = $cfg->{$field}->{'rows'} * $row_height;
			my $style = qq{style="height:${height}px;width:$cfg->{$field}->{'width'}px;"};
			$new =~ s#><TMPL_VAR# $style><TMPL_VAR#;
		} else {
			$new =~ /tabindex="(\d+)"/;
			my $tabindex = $1;
			my $tab = $tabindex ? qq{tabindex="$tabindex"} : '';
			if ($cfg->{$field}->{'type'} eq 'checkbox') {
				my $checked = qq{<TMPL_IF NAME=$ucfield> checked="checked"</TMPL_IF>};
				my $cb = qq{<input type="checkbox" value="1" name="$field" $tab$checked />};
				$new = '';
				$new_label = "$cb $new_label";
			} elsif ($cfg->{$field}->{'type'} eq 'date') {
				$new = <<"TMPL";
<script type="text/javascript" src="<TMPL_VAR NAME=STATIC_URI>plugins/RightFields/cal.js"></script>
<script type="text/javascript">
var rf_cal_$field = new CalendarPopup();
rf_cal_$field.setReturnFunction("rf_cal_set_$field");
function rf_cal_set_$field(y,m,d) {
	rf_cal_set('$field', y, m, d);
}
</script>
TMPL
				my $cal = <<"TMPL";
<a href="#" onclick="rf_cal_$field.showCalendar('rf_cal_a_$field');return false;" name="rf_cal_a_$field" id="rf_cal_a_$field"><img src="<TMPL_VAR NAME=STATIC_URI>plugins/RightFields/cal.png" border="0" /></a>
TMPL
				my $y = choice_tmpl("${field}_y", "RF_${ucfield}_Y", 'Year', 
					'y', $tabindex);
				my $m = choice_tmpl("${field}_m", "RF_${ucfield}_M", 'Month', 
					'm', $tabindex);
				my $d = choice_tmpl("${field}_d", "RF_${ucfield}_D", 'Day', 
					'd', $tabindex);
				my $hms = '';
				if ($cfg->{$field}->{'date_show_hms'}) {
					my $h = choice_tmpl("${field}_h", "RF_${ucfield}_H", 'HH',
						'h', $tabindex);
					my $min = ($cfg->{$field}->{'date_time'} ne 'hh')
						? ' : ' .choice_tmpl("${field}_min",
						"RF_${ucfield}_MIN", 'MM', 'min', $tabindex)
						: '';
					my $s = ($cfg->{$field}->{'date_time'} eq 'hhmmss')
						? ' : ' . choice_tmpl("${field}_s", "RF_${ucfield}_S",
						'SS', 's', $tabindex)
						: '';
					$hms = "$h$min$s";
				}
				if ($cfg->{$field}->{'date_order'} eq 'ymd') {
					$new .= "$y $m $d $cal $hms";
				} elsif ($cfg->{$field}->{'date_order'} eq 'dmy') {
					$new .= "$d $m $y $cal $hms";
				} else {
					$new .= "$m $d $y $cal $hms";
					if ($cfg->{$field}->{'date_ampm'}) {
						$new .= <<"TMPL";
<select name="${field}_ampm" id="${field}_ampm">
<option value="am"<TMPL_IF NAME=${ucfield}_AM_SELECTED> selected="selected"</TMPL_IF>>am</option>
<option value="pm"<TMPL_IF NAME=${ucfield}_PM_SELECTED> selected="selected"</TMPL_IF>>pm</option>
</select>
TMPL
					}
				}
			} elsif ($cfg->{$field}->{'type'} eq 'file') {
				$new = <<"TMPL";
<TMPL_IF NAME=RF_${ucfield}_IMAGE><table cellspacing="0" cellpadding="0" border="0"><tr valign="top"><td nowrap="nowrap"></TMPL_IF>
<input id="$field" name="$field" value="<TMPL_VAR NAME=$ucfield>" style="width:300px;" $tab /><TMPL_IF NAME=$ucfield><TMPL_UNLESS NAME=RF_${ucfield}_IMAGE>&nbsp;&nbsp;<a href="<TMPL_VAR NAME=RF_${ucfield}_URL_PATH>/<TMPL_VAR NAME=$ucfield>" target="_blank"><TMPL_VAR NAME=$ucfield></a></TMPL_UNLESS></TMPL_IF>
<TMPL_IF NAME=CAN_UPLOAD><br /><input type="file" name="${field}_upload" $tab /></TMPL_IF>
<TMPL_IF NAME=RF_${ucfield}_IMAGE></td><td><a href="<TMPL_VAR NAME=RF_${ucfield}_URL_PATH>/<TMPL_VAR NAME=$ucfield>" target="_blank"><img src="<TMPL_VAR NAME=RF_${ucfield}_URL_PATH>/<TMPL_VAR NAME=$ucfield>" height="50" border="0" hspace="10" /></a></td></tr></table></TMPL_IF>
TMPL
			} elsif ($cfg->{$field}->{'type'} eq 'menu') {
				$new = <<"TMPL";
<select name="$field" $tab>
<option value="">Select</option>
<TMPL_LOOP NAME=RF_${ucfield}_CHOICES>
<option value="<TMPL_VAR NAME=VALUE>"<TMPL_IF NAME=SELECTED> selected="selected"</TMPL_IF>><TMPL_VAR NAME=LABEL></option>
</TMPL_LOOP></select>
TMPL
			} elsif ($cfg->{$field}->{'type'} eq 'radio') {
				$new = <<"TMPL";
<TMPL_LOOP NAME=RF_${ucfield}_CHOICES>
<input type="radio" name="$field" value="<TMPL_VAR NAME=VALUE>"<TMPL_IF NAME=SELECTED> checked="checked"</TMPL_IF> $tab /> <TMPL_VAR NAME=LABEL>
&nbsp;
</TMPL_LOOP>
TMPL
			} elsif ($cfg->{$field}->{'type'} eq 'entry') {
				$new = <<"TMPL";
<select name="$field" $tab>
<option value="">Select</option>
<TMPL_LOOP NAME=RF_${ucfield}_ENTRIES>
<option value="<TMPL_VAR NAME=ID>"<TMPL_IF NAME=SELECTED> selected="selected"</TMPL_IF>><TMPL_VAR NAME=TITLE> [<TMPL_VAR NAME=ID>]</option>
</TMPL_LOOP></select>
TMPL
			}
		}
		$old = quotemeta($old);
		$$template =~ s/$old/$new/;
		$old_label = quotemeta($old_label);
		$$template =~ s#$old_label</label>\s+<a [^>]+>\?</a>#$new_label</label>#;
	}
}

sub template_extra {
	my ($cfg, $template) = @_;
	if ($cfg->{'datasource'}) {
		my $old = q{<TMPL_IF NAME=DISP_PREFS_SHOW_TAGS>};
		my $y = choice_tmpl('<TMPL_VAR NAME=NAME>_y', 'Y_LOOP', 'Year', 'y');
		my $m = choice_tmpl('<TMPL_VAR NAME=NAME>_m', 'M_LOOP', 'Month', 'm');
		my $d = choice_tmpl('<TMPL_VAR NAME=NAME>_d', 'D_LOOP', 'Day', 'd');
		my $h = choice_tmpl('<TMPL_VAR NAME=NAME>_h', 'H_LOOP', 'HH', 'h');
		my $min = choice_tmpl('<TMPL_VAR NAME=NAME>_min', 'MIN_LOOP', 'MM', 'min');
		my $s = choice_tmpl('<TMPL_VAR NAME=NAME>_s', 'S_LOOP', 'SS', 's');
		my $buttons = field_buttons_formatting() . field_buttons_resizing();
		$buttons =~ s/__FIELD__/<TMPL_VAR NAME=NAME>/g;
		my $buttons_top = <<"HTML";
<div class="field-wrapper">
<div class="field-buttons">
$buttons
</div>
HTML
		my $buttons_bottom = <<"HTML";
<input type="hidden" name="<TMPL_VAR NAME=NAME>_height" id="<TMPL_VAR NAME=NAME>_height" value="<TMPL_VAR NAME=HEIGHT>" />
</div>
HTML
		my $new = <<TMPL;
<TMPL_LOOP NAME=RF_LOOP>
<div class="field">
<TMPL_IF NAME=TYPE_TEXT>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<div class="textarea-wrapper" style="width:<TMPL_VAR NAME=WIDTH>px;">
<input id="<TMPL_VAR NAME=NAME>" name="<TMPL_VAR NAME=NAME>" value="<TMPL_VAR NAME=VALUE>" style="width:<TMPL_VAR NAME=WIDTH>px;"<TMPL_IF NAME=LENGTH> maxlength="<TMPL_VAR NAME=LENGTH>"</TMPL_IF> tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()"<TMPL_VAR NAME=GUIDE_JS> />
</div>
</TMPL_IF>
<TMPL_IF NAME=TYPE_TEXTAREA>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<TMPL_IF NAME=SHOW_BUTTONS>$buttons_top</TMPL_IF>
<div class="textarea-wrapper" style="width:<TMPL_VAR NAME=WIDTH>px;">
<textarea style="width:<TMPL_VAR NAME=WIDTH>px;" name="<TMPL_VAR NAME=NAME>" id="<TMPL_VAR NAME=NAME>" rows="<TMPL_VAR NAME=ROWS>" tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()"<TMPL_VAR NAME=GUIDE_JS>><TMPL_VAR NAME=VALUE></textarea>
</div>
<TMPL_IF NAME=SHOW_BUTTONS>$buttons_bottom</TMPL_IF>
</TMPL_IF>
<TMPL_IF NAME=TYPE_CHECKBOX>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><input type="checkbox" value="1" name="<TMPL_VAR NAME=NAME>" id="<TMPL_VAR NAME=NAME>"<TMPL_IF NAME=VALUE> checked="checked"</TMPL_IF> tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()" /> <TMPL_VAR NAME=LABEL></label></div>
</TMPL_IF>
<TMPL_IF NAME=TYPE_DATE>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<TMPL_IF NAME=YMD_SELECTED>$y $m $d</TMPL_IF>
<TMPL_IF NAME=MDY_SELECTED>$m $d $y</TMPL_IF>
<TMPL_IF NAME=DMY_SELECTED>$d $m $y</TMPL_IF>
<script type="text/javascript" src="<TMPL_VAR NAME=STATIC_URI>plugins/RightFields/cal.js"></script>
<script type="text/javascript">
var rf_cal_<TMPL_VAR NAME=NAME> = new CalendarPopup();
rf_cal_<TMPL_VAR NAME=NAME>.setReturnFunction("rf_cal_set_<TMPL_VAR NAME=NAME>");
function rf_cal_set_<TMPL_VAR NAME=NAME>(y,m,d) {
	rf_cal_set('<TMPL_VAR NAME=NAME>', y, m, d);
}
</script>
<a href="#" onclick="rf_cal_<TMPL_VAR NAME=NAME>.showCalendar('rf_cal_a_<TMPL_VAR NAME=NAME>');return false;" name="rf_cal_a_<TMPL_VAR NAME=NAME>" id="rf_cal_a_<TMPL_VAR NAME=NAME>"><img src="<TMPL_VAR NAME=STATIC_URI>plugins/RightFields/cal.png" border="0" /></a>
<TMPL_IF NAME=DATE_SHOW_HMS>
$h <TMPL_IF NAME=DATE_SHOW_MIN>: $min</TMPL_IF> <TMPL_IF NAME=DATE_SHOW_SEC>: $s</TMPL_IF>
<TMPL_IF NAME=DATE_AMPM>
<select name="<TMPL_VAR NAME=NAME>_ampm" id="<TMPL_VAR NAME=NAME>_ampm">
<option value="am"<TMPL_IF NAME=AM_SELECTED> selected="selected"</TMPL_IF>>am</option>
<option value="pm"<TMPL_IF NAME=PM_SELECTED> selected="selected"</TMPL_IF>>pm</option>
</select>
</TMPL_IF>
</TMPL_IF>
</TMPL_IF>
<TMPL_IF NAME=TYPE_FILE>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<TMPL_IF NAME=IMAGE>
<table cellspacing="0" cellpadding="0" border="0"><tr valign="top"><td nowrap="nowrap"></TMPL_IF>
<input id="<TMPL_VAR NAME=NAME>" name="<TMPL_VAR NAME=NAME>" value="<TMPL_VAR NAME=VALUE>" style="width:300px;" tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()" /><TMPL_IF NAME=VALUE><TMPL_UNLESS NAME=IMAGE>&nbsp;&nbsp;<a href="<TMPL_VAR NAME=URL_PATH>/<TMPL_VAR NAME=VALUE>" target="_blank"><TMPL_VAR NAME=VALUE></a></TMPL_UNLESS></TMPL_IF>
<TMPL_IF NAME=CAN_UPLOAD><br /><input type="file" name="<TMPL_VAR NAME=NAME>_upload" tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()" /></TMPL_IF>
<TMPL_IF NAME=IMAGE></td><td><a href="<TMPL_VAR NAME=URL_PATH>/<TMPL_VAR NAME=VALUE>" target="_blank"><img src="<TMPL_VAR NAME=URL_PATH>/<TMPL_VAR NAME=VALUE>" height="50" border="0" hspace="10" /></a></td></tr></table></TMPL_IF>
</TMPL_IF>
<TMPL_IF NAME=TYPE_MENU>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<select name="<TMPL_VAR NAME=NAME>" id="<TMPL_VAR NAME=NAME>" onchange="setDirty()">
<option value="">Select</option>
<TMPL_LOOP NAME=CHOICES_LOOP>
<option value="<TMPL_VAR NAME=VALUE>"<TMPL_IF NAME=SELECTED> selected="selected"</TMPL_IF>><TMPL_VAR NAME=LABEL></option>
</TMPL_LOOP></select>
</TMPL_IF>
<TMPL_IF NAME=TYPE_RADIO>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<TMPL_LOOP NAME=CHOICES_LOOP>
<input type="radio" name="<TMPL_VAR NAME=NAME>" value="<TMPL_VAR NAME=VALUE>"<TMPL_IF NAME=SELECTED> checked="checked"</TMPL_IF> tabindex="<TMPL_VAR NAME=TABINDEX>" onchange="setDirty()" /> <TMPL_VAR NAME=LABEL>
&nbsp;
</TMPL_LOOP>
</TMPL_IF>
<TMPL_IF NAME=TYPE_ENTRY>
<div class="field-header"><label for="<TMPL_VAR NAME=NAME>"><TMPL_VAR NAME=LABEL></label></div>
<select name="<TMPL_VAR NAME=NAME>" onchange="setDirty()">
<option value="">Select</option>
<TMPL_LOOP NAME=ENTRIES>
<option value="<TMPL_VAR NAME=ID>"<TMPL_IF NAME=SELECTED> selected="selected"</TMPL_IF>><TMPL_VAR NAME=TITLE> [<TMPL_VAR NAME=ID>]</option>
</TMPL_LOOP></select>
</TMPL_IF>
</div>
</TMPL_LOOP>

<TMPL_IF NAME=DISP_PREFS_SHOW_TAGS>
TMPL
		$old = quotemeta($old);
		$$template =~ s/$old/$new/s;
		for my $i (7 .. 12) {
			$$template =~ s/tabindex="$i"/tabindex="<TMPL_VAR NAME=TABINDEX_$i>"/g;
		}
	}
}

sub edit_entry_param {
	my ($cb, $app, $param) = @_;
	my $q = $app->{'query'};
	my $has_file = 0;
	my $has_date = 0;
	my $blog_id = $q->param('blog_id');
	my $defaults;
	if (!$q->param('id')) {
		$defaults = blog_data('defaults', $blog_id);
	}
	
	my $cfg = blog_data('standard', $blog_id);
	if (rf_enabled($cfg, $q)) {
		if (!$q->param('id')) {
				# new entry, populate with defaults
			if ($defaults) {
				my $fields = mt_fields();
				for my $field (keys %$fields) {
					my $key = 'std_' . $field;
					next unless ($defaults->{$key});
					if (($cfg->{$field}->{'type'} eq 'date')
						&& ($defaults->{$key} eq 'current')) {
						$defaults->{$key} = current_ts($blog_id);
					}
					$param->{$field} = $defaults->{$key};
					if ($defaults->{"${key}_is_guide"}) {
						$param->{"${field}_is_guide"} = 1;
					}
				}
			}
		}
		param_standard($cfg, $q, $param, \$has_file, \$has_date);
	}
	$cfg = blog_data('extra', $blog_id) || {};
	if ($cfg->{'datasource'} && rf_enabled($cfg, $q)) {
		if (!$q->param('id')) {
				# new entry, populate with defaults
			if ($defaults) {
				for my $field (keys %{$cfg->{'cols'}}) {
					next unless ($defaults->{$field});
					if (($cfg->{'cols'}->{$field}->{'type'} eq 'date')
						&& ($defaults->{$field} eq 'current')) {
						$defaults->{$field} = current_ts($blog_id);
					}
					$q->param(fld_name($field), $defaults->{$field});
					if ($defaults->{"${field}_is_guide"}) {
						$q->param(fld_name($field) . '_is_guide', 1);
					}
				}
			}
		}
		param_extra($cfg, $q, $param, \$has_file, \$has_date, $defaults);
	}
	$param->{'rf_has_file'} = $has_file;
	$param->{'rf_has_date'} = $has_date;
}

sub param_standard {
	my ($cfg, $q, $param, $has_file, $has_date) = @_;
	
	my $fields = mt_fields();
	for my $field (keys %$fields) {
		next unless ($cfg && %$cfg && $cfg->{$field}->{'customize'});
		$param->{$field} ||= '';
		if ($cfg->{$field}->{'type'} eq 'date') {
			$$has_date++;
			my ($y, $m, $d, $h, $min, $s) = unpack('A4A2A2A2A2A2', 
				$param->{$field});
			if ($cfg->{$field}->{'date_ampm'} && $h) {
				if ($h >= 12) {
					$param->{"${field}_pm_selected"} = 1;
				} else {
					$param->{"${field}_am_selected"} = 1;
				}
				if ($h > 12) {
					$h = $h - 12;
				}
			}
			$param->{"rf_${field}_y"} = option_loop(
				$cfg->{$field}->{'date_y_start'},
				$cfg->{$field}->{'date_y_end'}, $y);
			$param->{"rf_${field}_m"} = option_loop(1, 12, $m);
			$param->{"rf_${field}_d"} = option_loop(1, 31, $d);
			if ($cfg->{$field}->{'date_show_hms'}) {
				my ($start_h, $end_h) = $cfg->{$field}->{'date_ampm'} 
					? (1, 12) : (0, 23);
				$param->{"rf_${field}_h"} = option_loop($start_h, $end_h, $h);
				if ($cfg->{$field}->{'date_time'} ne 'hh') {
					$param->{"rf_${field}_min"} = option_loop_step(0, 59,
						$cfg->{$field}->{'date_minutes'}, $min);
				}
				if ($cfg->{$field}->{'date_time'} eq 'hhmmss') {
					$param->{"rf_${field}_s"} = option_loop(0, 59, $s);
				}
			}
		} elsif ($cfg->{$field}->{'type'} eq 'file') {
			$$has_file++;
			for my $key (qw( upload_path url_path )) {
				$param->{"rf_${field}_$key"} = $cfg->{$field}->{$key};
			}
			if ($param->{$field} && $param->{$field} =~ /\.(jpg|gif|bmp|png|jpeg)$/) {
				$param->{"rf_${field}_image"} = 1;
			}
		} elsif ($cfg->{$field}->{'type'} =~ /^menu|radio$/) {
			my @choices_loop;
			for my $choice (split(/[\n\r]+/, $cfg->{$field}->{'choices'})) {
				next unless $choice;
				my ($value, $label) = split(/=/, $choice);
				$label ||= $value;
				push(@choices_loop, {
					'value' => $value,
					'selected' => ($param->{$field} && 
						(($param->{$field} eq $value) || 
							$param->{$field} eq $label)) ? 1 : 0,
					'label' => $label
				});
			}
			$param->{"rf_${field}_choices"} = \@choices_loop;
		} elsif ($cfg->{$field}->{'type'} eq 'entry') {
			next unless ($cfg->{$field}->{'weblog'});
			my @rows = ();
			my @entries = MT::Entry->load({
				'blog_id' => $cfg->{$field}->{'weblog'}
			});
			my @cats = ();
			if ($cfg->{$field}->{'category_ids'}) {
				require MT::Category;
				for my $cat_id (split(/,/, $cfg->{$field}->{'category_ids'})) {
					push(@cats, MT::Category->load($cat_id));
				}
			}
			for my $entry (sort { $a->title cmp $b->title } @entries) {
				if (@cats) {
					my $proceed = 0;
					for my $cat (@cats) {
						if ($entry->is_in_category($cat)) {
							$proceed = 1;
							last;
						}
					}
					next unless $proceed;
				}
				push (@rows, {
					'id' => $entry->id,
					'title' => remove_html($entry->title),
					'selected' => $param->{$field} ?
						(($param->{$field} == $entry->id) ? 1 : 0) : 0
				});
			}
			$param->{"rf_${field}_entries"} = \@rows;
		}
	}
}

sub param_extra {
	my ($cfg, $q, $param, $has_file, $has_date, $defaults) = @_;
	my @rows = ();
	my $e_data;
	if ($q->param('id')) {
		$e_data = new_obj($cfg, $q->param('id'));
	}
	my $tabindex = 6;
	for my $field (sort { $cfg->{'cols'}->{$a}->{'sort'} <=> 
			$cfg->{'cols'}->{$b}->{'sort'} } keys %{$cfg->{'cols'}}) {
		my $field_cfg = $cfg->{'cols'}->{$field};
		next if ($field_cfg->{'ignore'});
		$tabindex++;
		my $row = { 'name' => fld_name($field),
			'label' => $field_cfg->{'label'},
			'value' => defined($q->param(fld_name($field)))
				? $q->param(fld_name($field)) : $e_data
					? $e_data->column($field) : '',
			'tabindex' => $tabindex
		};
		$row->{"type_$field_cfg->{'type'}"} = 1;
		if (!$param->{'id'} && ($field_cfg->{'type'} =~ /^text/)) {
			if ($defaults && $defaults->{$field . '_is_guide'}) {
				$row->{'guide_js'} = guide_js(fld_name($field), 
					$row->{'value'});
			}
		}
		if ($field_cfg->{'type'} eq 'text') {
			$row->{'width'} = $field_cfg->{'width'};
			$row->{'length'} = $field_cfg->{'length'};
			require MT::Util;
			$row->{'value'} = MT::Util::encode_html($row->{'value'});
		} elsif ($field_cfg->{'type'} eq 'textarea') {
			$row->{'width'} = $field_cfg->{'width'};
			$row->{'rows'} = $field_cfg->{'rows'};
			$row->{'height'} = $field_cfg->{'rows'} * $row_height;
			unless (defined($field_cfg->{'show_buttons'})
					&& ($field_cfg->{'show_buttons'} == 0)) {
				$row->{'show_buttons'} = 1;
			}
		} elsif ($field_cfg->{'type'} eq 'date') {
			$$has_date++;
			$field_cfg->{'date_time'} ||= 'hhmm';
			$field_cfg->{'date_minutes'} ||= 1;
			for my $key (qw( date_show_hms date_ampm )) {
				$row->{$key} = $field_cfg->{$key};
			}
			$row->{'date_show_min'} = ($field_cfg->{'date_time'} ne 'hh')
				? 1 : 0;
			$row->{'date_show_sec'} = ($field_cfg->{'date_time'} eq 'hhmmss')
				? 1 : 0;
			$field_cfg->{'date_order'} ||= 'mdy';
			$row->{$field_cfg->{'date_order'} . '_selected'} = 1;
			$tabindex--;
			for (split(//, $field_cfg->{'date_order'})) {
				$row->{"tabindex_$_"} = ++$tabindex;
			}
			$tabindex += 2;
				# numerify it in case the field type was changed
				# and there's a text value in there
			$row->{'value'} = eval("$row->{'value'} + 0") || '';
			my ($y, $m, $d, $h, $min, $s) = unpack('A4A2A2A2A2A2', 
				$row->{'value'});
			if ($field_cfg->{'date_ampm'} && $h) {
				if ($h >= 12) {
					$row->{'pm_selected'} = 1;
				} else {
					$row->{'am_selected'} = 1;
				}
				if ($h > 12) {
					$h = $h - 12;
				}
			}
			$row->{'y_loop'} = option_loop($field_cfg->{'date_y_start'},
				$field_cfg->{'date_y_end'}, $y);
			$row->{'m_loop'} = option_loop(1, 12, $m);
			$row->{'d_loop'} = option_loop(1, 31, $d);
			if ($field_cfg->{'date_show_hms'}) {
				my ($start_h, $end_h) = $field_cfg->{'date_ampm'}
					? (1, 12) : (0, 23);
				$row->{'h_loop'} = option_loop($start_h, $end_h, $h);
				$row->{"tabindex_h"} = ++$tabindex;
				if ($field_cfg->{'date_time'} ne 'hh') {
					$row->{'min_loop'} = option_loop_step(0, 59,
						$field_cfg->{'date_minutes'}, $min);
					$row->{"tabindex_min"} = ++$tabindex;					
				}
				if ($field_cfg->{'date_time'} eq 'hhmmss') {
					$row->{'s_loop'} = option_loop(0, 59, $s);
					$row->{"tabindex_s"} = ++$tabindex;
				}
			}
		} elsif ($field_cfg->{'type'} eq 'file') {
			for my $key (qw( upload_path url_path overwrite )) {
				$row->{$key} = $field_cfg->{$key};
				next unless $row->{$key};
					# strip trailing slash
				$row->{$key} =~ s#/$##;
			}
			if ($row->{'value'} && $row->{'value'} =~ /\.(jpg|gif|bmp|png|jpeg)$/) {
				$row->{'image'} = 1;
			}
			$$has_file++;
		} elsif ($field_cfg->{'type'} =~ /^menu|radio$/) {
			my @choices_loop;
			for my $choice (split(/[\n\r]+/, $field_cfg->{'choices'})) {
				my ($value, $label) = split(/=/, $choice);
				$label ||= $value;
				push(@choices_loop, {
					'value' => $value,
					'selected' => ($row->{'value'} && 
						(($row->{'value'} eq $value) || 
							($row->{'value'} eq $label))) ? 1 : 0,
					'label' => $label,
					'name' => $row->{'name'},
					($field_cfg->{'type'} eq 'radio') ? 
						('tabindex' => $tabindex) : ()
				});
			}
			
			$row->{'choices_loop'} = \@choices_loop;
		} elsif ($field_cfg->{'type'} eq 'entry') {
			next unless ($field_cfg->{'weblog'});
			my @entry_rows = ();
			my @entries = MT::Entry->load({
				'blog_id' => $field_cfg->{'weblog'}
			});
			my @cats = ();
			if ($field_cfg->{'category_ids'}) {
				require MT::Category;
				for my $cat_id (split(/,/,
						$field_cfg->{'category_ids'})) {
					push(@cats, MT::Category->load($cat_id));
				}
			}
			for my $entry (sort { $a->title cmp $b->title } @entries) {
				if (@cats) {
					my $proceed = 0;
					for my $cat (@cats) {
						if ($entry->is_in_category($cat)) {
							$proceed = 1;
							last;
						}
					}
					next unless $proceed;
				}
				push (@entry_rows, {
					'id' => $entry->id,
					'title' => remove_html((length($entry->title) > 80)
						? substr($entry->title, 0, 80) . '...'
						: $entry->title),
					'selected' => (($row->{'value'} || 0) == $entry->id)
				});
			}
			$row->{'entries'} = \@entry_rows;
		}
		push(@rows, $row);
	}
	$param->{'rf_loop'} = \@rows;
	for my $i (7 .. 12) {
		$param->{"tabindex_$i"} = ++$tabindex;
	}
}

sub fields_html {
	return {
		'title' => qq{<input name="title" id="title" tabindex="1" value="<TMPL_VAR NAME=TITLE ESCAPE=HTML>" maxlength="255" <TMPL_IF NAME=DISP_PREFS_SHOW_BASENAME>onchange="rebasename(this)"</TMPL_IF> />},
		'text' => qq{<div class="textarea-wrapper">
<textarea onkeypress="mtShortCuts(event)" name="text" id="text" tabindex="<TMPL_IF NAME=DISP_PREFS_BASIC>2<TMPL_ELSE>3</TMPL_IF>" cols="72" rows="<TMPL_IF NAME=DISP_PREFS_HEIGHT_BODY>15<TMPL_ELSE><TMPL_IF NAME=DISP_PREFS_SHOW_EXTENDED>10<TMPL_ELSE>20</TMPL_IF></TMPL_IF>" style="height: <TMPL_IF NAME=DISP_PREFS_HEIGHT_BODY><TMPL_VAR NAME=DISP_PREFS_HEIGHT_BODY><TMPL_ELSE><TMPL_IF NAME=DISP_PREFS_SHOW_EXTENDED>130<TMPL_ELSE>194</TMPL_IF></TMPL_IF>px" onchange="setDirty()"><TMPL_VAR NAME=TEXT ESCAPE=HTML></textarea>
</div>},
		'text_more' => qq{<div class="textarea-wrapper">
<textarea onkeypress="mtShortCuts(event)" name="text_more" id="text_more" tabindex="4" cols="72" rows="15" style="height: <TMPL_IF NAME=DISP_PREFS_HEIGHT_EXTENDED><TMPL_VAR NAME=DISP_PREFS_HEIGHT_EXTENDED><TMPL_ELSE>162</TMPL_IF>px" onchange="setDirty()"><TMPL_VAR NAME=TEXT_MORE ESCAPE=HTML></textarea>
</div>},
		'excerpt' => qq{<div class="textarea-wrapper">
<textarea name="excerpt" id="excerpt" tabindex="5" cols="72" rows="5" onchange="setDirty()"><TMPL_VAR NAME=EXCERPT ESCAPE=HTML></textarea>
</div>},
		'keywords' => qq{<div class="textarea-wrapper">
<textarea name="keywords" id="keywords" tabindex="6" cols="72" rows="2" onchange="setDirty()"><TMPL_VAR NAME=KEYWORDS ESCAPE=HTML></textarea>
</div>}
	};
}

sub field_buttons_resizing {
	return q#<div class="field-buttons-resizing">
<script type="text/javascript">
with (document) {
    write('<img title="<MT_TRANS phrase="Bigger">" onclick="return expandEditor( \'__FIELD__\', \'expand\' );" src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/field-bigger.gif" alt="<MT_TRANS phrase="Bigger">" width="26" height="19" /></a>');
    write('<img title="<MT_TRANS phrase="Smaller">" onclick="return expandEditor( \'__FIELD__\', \'contract\' );" src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/field-smaller.gif" alt="<MT_TRANS phrase="Smaller">" width="26" height="19" /></a>');
}
</script>
</div>#;
}


sub field_buttons_formatting {
	return q[<div class="field-buttons-formatting">
<script type="text/javascript">
if (canFormat) {
    with (document) {
        write('<a title="<MT_TRANS phrase="Bold">" href="#" onclick="return formatStr(document.entry_form.__FIELD__, \'strong\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/bold.gif" alt="<MT_TRANS phrase="Bold">" width="26" height="19" /></a>');
        write('<a title="<MT_TRANS phrase="Italic">" href="#" onclick="return formatStr(document.entry_form.__FIELD__, \'em\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/italic.gif" alt="<MT_TRANS phrase="Italic">" width="26" height="19" /></a>');
        write('<a title="<MT_TRANS phrase="Underline">" href="#" onclick="return formatStr(document.entry_form.__FIELD__, \'u\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/underline.gif" alt="<MT_TRANS phrase="Underline">" width="26" height="19" /></a>');
        write('<a title="<MT_TRANS phrase="Insert Link">" href="#" onclick="return insertLink(document.entry_form.__FIELD__)"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/link.gif" alt="<MT_TRANS phrase="Insert Link">" width="26" height="19" /></a>');
        write('<a title="<MT_TRANS phrase="Insert Email Link">" href="#" onclick="return insertLink(document.entry_form.__FIELD__, 1)"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/email.gif" alt="<MT_TRANS phrase="Insert Email Link">" width="26" height="19" /></a>');
        write('<a title="<MT_TRANS phrase="Quote">" href="#" onclick="return formatStr(document.entry_form.__FIELD__, \'blockquote\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/quote.gif" alt="<MT_TRANS phrase="Quote">" width="26" height="19" /></a>');
    }
}
</script>
</div>];
}

sub preview_entry_param {
	my ($cb, $app, $param) = @_;
	my $q = $app->{'query'};
	my $cfg = blog_data('extra', $q->param('blog_id'));
	if ($cfg && %$cfg && rf_enabled($cfg, $q)) {
		my $value;
		for my $col (keys %{$cfg->{'cols'}}) {
			my $field = fld_name($col);
			if ($cfg->{'cols'}->{$col}->{'type'} eq 'date') {
				$value = params_to_ts($q, $field);
			} else {
				$value = $q->param($field) || '';
			}
			my $input = { 'data_name' => $field,
				'data_value' => $value
			};
			push(@{$param->{'entry_loop'}}, $input);
		}
	}
	$cfg = blog_data('standard', $q->param('blog_id'));
	if ($cfg && %$cfg && rf_enabled($cfg, $q)) {
		my $fields = mt_fields();
		for my $field (keys %$fields) {
			next unless ($cfg->{$field}->{'customize'});
			if ($cfg->{$field}->{'type'} eq 'date') {
				for my $input (@{$param->{'entry_loop'}}) {
					if ($input->{'data_name'} eq $field) {
						$input->{'data_value'} = params_to_ts($q, $field);
						last;
					}
				}
			}
		}
	}
}

sub list_entry_param {
	my ($cb, $app, $param) = @_;
	my $q = $app->{'query'};
	my $cfg = blog_data('listing', $q->param('blog_id'));
	return unless ($cfg && %$cfg);
	my $tmpl_text;
	if ($cfg->{'customize_compact'} && !$param->{'view_expanded'}) {
		$tmpl_text = $cfg->{'compact'};
	} elsif ($cfg->{'customize_expanded'} && $param->{'view_expanded'}) {
		$tmpl_text = $cfg->{'expanded'};
	}
	return unless $tmpl_text;
	if ($tmpl_text !~ /<MTExtraFields/) {
		$tmpl_text = "<MTExtraFields>$tmpl_text</MTExtraFields>";
	}
	require MT::Template;
	require MT::Template::Context;
	require MT::Entry;
	my $tmpl = MT::Template->new;
	$tmpl->text($tmpl_text);
	for my $row (@{$param->{'entry_table'}->[0]->{'object_loop'}}) {
		my $ctx = MT::Template::Context->new;
		$ctx->stash('entry', MT::Entry->load($row->{'id'}));
		$row->{'rf_listing_text'} = $tmpl->build($ctx);
	}
}

sub entry_table_template {
	my ($cb, $app, $template) = @_;
	my $q = $app->{'query'};
	my $cfg = blog_data('listing', $q->param('blog_id'));
	return unless ($cfg && %$cfg);
	my $new = '<TMPL_VAR NAME=RF_LISTING_TEXT>';
	if ($cfg->{'customize_compact'}) {
		my $old = '<TMPL_VAR NAME=TITLE_SHORT>';
		$old = quotemeta($old);
		$$template =~ s/$old/$new/;
	}
	if ($cfg->{'customize_expanded'}) {
		my $old = q{<strong><TMPL_VAR NAME=TITLE_LONG></strong><TMPL_IF NAME=EXCERPT> &mdash; <TMPL_VAR NAME=EXCERPT></TMPL_IF>};
		$old = quotemeta($old);
		$$template =~ s/$old/$new/;
	}
}

sub rf_enabled {
	my ($cfg, $q, $entry) = @_;
	return 0 unless ($cfg && %$cfg);
		# if it's not set, settings are from an older version so assume enabled
	return 1 unless $cfg->{'enabled'};
	return 0 if ($cfg->{'enabled'} == -1);
	if ($cfg->{'enabled'} == 2) {
			# all entries after...
			# assume enabled if this is not set for some reason
		return 1 unless $cfg->{'enabled_ts'};
			# new entry
		return 1 unless $q->param('id');
		my $e ||= MT::Entry->load($q->param('id'));
		return 1 unless $e;
		return 0 if ($e->created_on le $cfg->{'enabled_ts'});
	}
	return 1;
}

sub choice_tmpl {
	my ($fieldname, $loopname, $firstitem, $suffix, $tabindex) = @_;
	if (!$tabindex) {
		$suffix = uc($suffix);
		$tabindex = "<TMPL_VAR NAME=TABINDEX_$suffix>";
	}
	return <<TMPL;
<select name="$fieldname" id="$fieldname" tabindex="$tabindex" onchange="setDirty()">
<option value="">$firstitem</option>
<TMPL_LOOP NAME=$loopname>
<option value="<TMPL_VAR NAME=VALUE>"<TMPL_IF NAME=SELECTED> selected="selected"</TMPL_IF>><TMPL_VAR NAME=LABEL></option>
</TMPL_LOOP>
</select>
TMPL
}

sub option_loop {
	my ($start, $end, $selected, $labels) = @_;
	$selected ||= 0;
	my @loop = ();
	for my $i ($start .. $end) {
		push(@loop, one_option($i, $selected, $labels));
	}
	return \@loop;
}

sub option_loop_step {
	my ($start, $end, $step, $selected, $labels) = @_;
	$selected ||= 0;
	my @loop = ();
	for my $i ($start .. $end) {
		next unless ($i % $step == 0);
		push(@loop, one_option($i, $selected, $labels));
	}
	return \@loop;
}

sub one_option {
	my ($i, $selected, $labels) = @_;
	return {
			'value' => $i,
			'selected' => ($i && eval("$i == $selected")) ? 1 : 0,
			'label' => $labels ? $labels->{$i} : 
				($i < 10) ? "0$i" : $i
	};
}

sub month_hash {
	return {
		1 => 'January', 2 => 'February', 3 => 'March', 4 => 'April',
		5 => 'May', 6 => 'June', 7 => 'July', 8 => 'August',
		9 => 'September', 10 => 'October', 11 => 'November', 12 => 'December'
	};
}

sub search_hit {
	my ($mt_search_hit, $app, $entry) = @_;
	return 1 if &{$mt_search_hit}($app, $entry);
	return 0 unless $app->{'searchparam'}{'SearchElement'} eq 'entries';
	my $q = $app->{'query'};
	my $blog_id = $entry->blog_id;
	my $cfg = blog_data('extra', $blog_id) || {};
	return 0 unless ($cfg->{'datasource'} && rf_enabled($cfg, $q, $entry));
	my $extra_fields = new_obj($cfg, $entry->id);
	return 0 unless $extra_fields;
	my $text = '';
	for my $key (keys %{$cfg->{'cols'}}) {
		next unless $extra_fields->$key;
		$text .= $extra_fields->$key . "\n";
	}
	return 1 if $app->is_a_match($text);
}

	# TEMPLATE TAG HANDLERS

sub hdlr_entries {
	my ($mt_hdlr_entries, $ctx, $args, $cond) = @_;
	require MT::Entry;
	if (!$args->{'sort_by'} || MT::Entry->can($args->{'sort_by'})) {
		return &{$mt_hdlr_entries}($ctx, $args, $cond);
	}
	my $cfg = blog_data('extra', $ctx->stash('blog_id'));
	{
	no strict "refs";
		# in order to get MTEntries to sort by an extra field, we use
		# the symbol table to give MT::Entry a subroutine with
		# the name of the extra field. (have to disable strict refs for this.)
		# so when MTEntries sorts by $a->$col cmp $b->$col, it's calling
		# our subroutine to get the values to sort by.
		# this should work transparently with SQL or PluginData.
	*{"MT::Entry::$args->{'sort_by'}"} = sub {
		my $entry = shift;
		my $extra_fields = new_obj($cfg, $entry->id);
		return '' unless $extra_fields;
		return $extra_fields->column($args->{'sort_by'}) || '';
	};
	}
	return &{$mt_hdlr_entries}($ctx, $args, $cond);
}

sub hdlr_custom_tag {
	my ($ctx, $args, $cond, $field) = @_;
	my $entry = $ctx->stash('entry')
		|| return $ctx->_no_entry_error($ctx->{'tag'});
	if ($field =~ /^std /) {
			# standard field
		$field =~ s/^std //;
		return $entry->$field;
	} else {
		$args->{'field'} = $field;
		return hdlr_extra_field_value($ctx, $args, $cond);
	}
}

sub hdlr_linked_entry {
	my ($ctx, $args, $cond) = @_;
	my $linked_id;
	if ($linked_id = $args->{'id'}) {
		($linked_id = build_value($ctx, $linked_id));
	} else {
		(my $field = $args->{'field'}) || return '';
		my $entry = $ctx->stash('entry')
			|| return $ctx->_no_entry_error('MTLinkedEntry');
			# standard entry field?
		if (mt_fields()->{$field}) {
			$linked_id = $entry->$field;
		} else {
				# extra field
			defined($ctx->stash('rf_extra_' . $entry->id))
				|| return $ctx->error('Not called from within MTExtraFields.');
			$linked_id = hdlr_extra_field_value(@_);
		}
	}
	return '' unless $linked_id;
	(my $linked_entry = MT::Entry->load($linked_id)) || return '';
	local $ctx->{'__stash'}{'entry'} = $linked_entry;
	local $ctx->{'current_timestamp'} = $linked_entry->created_on;
	local $ctx->{'modification_timestamp'} = $linked_entry->modified_on;
	defined(my $text = $ctx->stash('builder')->build($ctx,
		$ctx->stash('tokens'), $cond)) || return $ctx->error($ctx->errstr);
	return $text;
}

sub hdlr_linking_entries {
	my ($ctx, $args, $cond) = @_;
	my $entry = $ctx->stash('entry')
		|| return $ctx->_no_entry_error('MTLinkingEntries');
	my $blog_ids = $args->{'blog_ids'} ||  $args->{'blog_id'} || '';
	my %blog_ids = map { $_ => 1 } split(/,/, $blog_ids);
	$args->{'field'} || return $ctx->error('No field passed.');
	my @entries;
	my $mt_fields = mt_fields();
	if ($mt_fields->{$args->{'field'}}) {
		@entries = MT::Entry->load({
			%blog_ids ? ('blog_id' => [ keys %blog_ids ]) : (),
			'status' => 2,
			$args->{'field'} => $entry->id
		});
	} else {
		my $cfg = blog_data('extra', %blog_ids ? ${[keys %blog_ids]}[0] : -1);
		if ($cfg->{'datasource'} eq '_pseudo') {
			return $ctx->error('MTLinkingEntries can only be used with a SQL datasource.');
		} else {
			require RightFieldsObject;
				# get it to load the column defs
			new_obj($cfg);
			my @ef_rows = RightFieldsObject->load({
				$args->{'field'} => $entry->id 
			});
			for my $row (@ef_rows) {
				my $linking_entry = MT::Entry->load($row->id);
				next if (%blog_ids && !$blog_ids{$linking_entry->blog_id});
				next if ($linking_entry->status != 2);
				push(@entries, $linking_entry);
			}
		}
	}
	return '' unless @entries;
	my $sort_col = $args->{'sort_by'} || 'created_on';
	$args->{'sort_order'} ||= '';
	my $desc;
	if ($sort_col eq 'title') {
		$desc = ($args->{'sort_order'} && ($args->{'sort_order'} eq 'descend'));
	} else {
		$desc = !($args->{'sort_order'} && ($args->{'sort_order'} eq 'ascend'));
	}
	if ($desc) {
		@entries = sort { $b->$sort_col cmp $a->$sort_col } @entries;
	} else {
		@entries = sort { $a->$sort_col cmp $b->$sort_col } @entries;
	}
	my $text = '';
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	for my $linking_entry (@entries) {
		my $blog = MT::Blog->load($linking_entry->blog_id);
		local $ctx->{'__stash'}{'blog'} = $blog;
		local $ctx->{'__stash'}{'blog_id'} = $linking_entry->blog_id;
		local $ctx->{'__stash'}{'entry'} = $linking_entry;
		local $ctx->{'current_timestamp'} = $linking_entry->created_on;
		local $ctx->{'modification_timestamp'} = $linking_entry->modified_on;
		defined(my $row = $builder->build($ctx, $tokens, $cond))
			|| return $ctx->error($ctx->errstr);
		$text .= $row;
	}
	return $text;
}

sub hdlr_if_linking_entries {
	my ($ctx, $args, $cond) = @_;
	my $entry = $ctx->stash('entry')
		|| return $ctx->_no_entry_error('MTIfLinkingEntries');
	my $blog_ids = $args->{'blog_ids'} ||  $args->{'blog_id'} || '';
	my %blog_ids = map { $_ => 1 } split(/,/, $blog_ids);
	$args->{'field'} || return $ctx->error('No field passed.');
	my $mt_fields = mt_fields();
	if ($mt_fields->{$args->{'field'}}) {
		 return MT::Entry->count({
			%blog_ids ? ('blog_id' => [ keys %blog_ids ]) : (),
			'status' => 2,
			$args->{'field'} => $entry->id
		});
	} else {
		my $cfg = blog_data('extra', %blog_ids ? ${[keys %blog_ids]}[0] : -1);
		if ($cfg->{'datasource'} eq '_pseudo') {
			return $ctx->error('MTIfLinkingEntries can only be used with a SQL datasource.');
		} else {
			require RightFieldsObject;
				# get it to load the column defs
			new_obj($cfg);
			if (!%blog_ids) {
				return RightFieldsObject->count({
					$args->{'field'} => $entry->id 
				});
			}
			my @ef_rows = RightFieldsObject->load({
				$args->{'field'} => $entry->id 
			});
			for my $row (@ef_rows) {
				my $linking_entry = MT::Entry->load($row->id);
				next if (%blog_ids && !$blog_ids{$linking_entry->blog_id});
				next if ($linking_entry->status != 2);
				return 1;
			}
		}
	}
}

sub hdlr_extra_fields {
	my ($ctx, $args, $cond) = @_;
	my $entry = $ctx->stash('entry')
		|| return $ctx->_no_entry_error('MTExtraFields');
	my $blog_id = $entry->blog_id;
	my $cfg = blog_data('extra', $blog_id);
	return $ctx->error("No extra fields are configured for blog ID $blog_id")
		unless $cfg;
		# load the row with the extra data
	local $ctx->{__stash}{'rf_extra_' . $entry->id} = new_obj($cfg, $entry->id);
	defined(my $text = $ctx->stash('builder')->build($ctx,
		$ctx->stash('tokens'), $cond)) || return $ctx->error($ctx->errstr);
	return $text;
}

sub hdlr_if_extra_fields {
# handler for MT tag that prints its contents if extra data has been loaded
# for the current entry
	my ($ctx, $args, $cond) = @_;
	(my $entry = $ctx->stash('entry'))
		|| return $ctx->_no_entry_error('MTIfExtraFields');
	defined (my $extra_fields = $ctx->stash('rf_extra_' . $entry->id))
		|| return $ctx->error('Not called from within MTExtraFields.');
	return 0 unless $extra_fields;
	my $cfg = blog_data('extra', $entry->blog_id);
		# make sure at least one of the fields contains something
	my %fields = $args->{'fields'} ? split(' ', $args->{'fields'})
		: ();
	for my $key (keys %{$cfg->{'cols'}}) {
		next if (%fields && !$fields{$key});
		if ($extra_fields->$key) {
			return 1;
		}
	}
	return 0;
}

sub hdlr_if_extra_field {
	return hdlr_extra_field_value(@_) ? 1 : 0;
}

sub hdlr_extra_fields_loop {
	my ($ctx, $args, $cond) = @_;
	(my $entry = $ctx->stash('entry'))
		|| return $ctx->_no_entry_error('MTExtraFieldsLoop');
	defined ($ctx->stash('rf_extra_' . $entry->id))
		|| return $ctx->error('Not called from within MTExtraFields.');
	my $cfg = blog_data('extra', $entry->blog_id);
	my $text = '';
		# store initial builder and tokens so we can repeat them
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	for my $key (sort { $cfg->{'cols'}->{$a}->{'sort'}
			<=> $cfg->{'cols'}->{$b}->{'sort'} }
			keys %{$cfg->{'cols'}}) {
		next if ($cfg->{'cols'}->{$key}->{'ignore'});
		local $ctx->{__stash}{'rf_col'} = $key;
		defined(my $iter = $builder->build($ctx, $tokens, $cond))
			|| return $ctx->error($ctx->errstr);
		$text .= $iter;
	}
	return $text;
}

sub hdlr_extra_field_value {
	my ($ctx, $args) = @_;
	(my $entry = $ctx->stash('entry'))
		|| return $ctx->_no_entry_error('MTExtraFieldValue');
	defined (my $extra_fields = $ctx->stash('rf_extra_' . $entry->id))
		|| return $ctx->error('Not called from within MTExtraFields.');
	return '' unless $extra_fields;
	my $cfg = blog_data('extra', $entry->blog_id);
	my $col = get_col($ctx, $args);
	my $text = $extra_fields->column($col) || '';
	if ($cfg->{'cols'}->{$col}->{'type'} =~ /^menu|radio$/) {
		return $text if ($args->{'show_key'});
		for my $choice (split(/\n/, $cfg->{'cols'}->{$col}->{'choices'})) {
			if ($choice =~ /^$text=/) {
				next unless $choice;
				(undef, $text) = split(/=/, $choice);
				return $text || '';
			}
		}
	}
	return $text unless ($cfg->{'cols'}->{$col}->{'type'} eq 'textarea');
	return $text unless ($cfg->{'cols'}->{$col}->{'text_filters'});
    my $convert_breaks = exists $args->{convert_breaks} ?
        $args->{convert_breaks} :
            defined $entry->convert_breaks ? $entry->convert_breaks :
                $ctx->stash('blog')->convert_paras;
    if ($convert_breaks) {
        my $filters = $entry->text_filters;
        push @$filters, '__default__' unless @$filters;
        $text = MT->apply_text_filters($text, $filters, $_[0]);
    }
    return $text;
}

sub hdlr_field_value {
	my ($ctx, $args) = @_;
	my $mt_fields = mt_fields();
	my $field = get_col($ctx, $args);
	if ($mt_fields->{$field}) {
		(my $entry = $ctx->stash('entry'))
			|| return $ctx->_no_entry_error('MTFieldValue');
		(my $field = $args->{'field'}) || return $ctx->error('No field passed.');
		(my $text = $entry->$field) || return '';
		my $cfg = blog_data('standard', $entry->blog_id);
		return $text unless $cfg;
		return $text unless ($cfg->{$field}->{'type'} =~ /^menu|radio$/);
		return $text if ($args->{'show_key'});
		for my $choice (split(/[\n\r]/, $cfg->{$field}->{'choices'})) {
			next unless $choice;
			if ($choice =~ /^$text=/) {
				(undef, $text) = split(/=/, $choice);
				return $text;
			}
		}
		return $text; # choice not found?
	} else {
		return hdlr_extra_field_value(@_);
	}
}

sub hdlr_field_key {
	my ($ctx, $args) = @_;
	$args->{'show_key'} = 1;
	return hdlr_field_value(@_);
}

sub hdlr_field_label {
	my ($ctx, $args) = @_;
	return get_elem($ctx, $args, 'label');
}

sub hdlr_file_url_path {
	my ($ctx, $args) = @_;
	my $path = get_elem($ctx, $args, 'url_path');
	$path =~ s#/$##;
	return $path;
}

sub get_elem {
	my ($ctx, $args, $elem) = @_;
	(my $entry = $ctx->stash('entry'))
		|| return $ctx->_no_entry_error($ctx->stash('tag'));
	my $mt_fields = mt_fields();
	my $field = get_col($ctx, $args);
	my ($cfg, $col);
	if ($mt_fields->{$field}) {
		$cfg = blog_data('standard', $entry->blog_id);
		$col = $cfg->{$field};
	} else {
		$cfg = blog_data('extra', $entry->blog_id);
		$col = $cfg->{'cols'}->{$field};
	}
	return $col->{$elem} || '';
}

sub get_col {
	my ($ctx, $args) = @_;
	my $col = $ctx->stash('rf_col') || $args->{'field'}
		|| return $ctx->error('No field passed');
	return $col;
}

sub hdlr_format_date {
	my ($ctx, $args, $cond) = @_;
	defined(my $text = $ctx->stash('builder')->build($ctx,
		$ctx->stash('tokens'), $cond)) || return $ctx->error($ctx->errstr);
		# strip whitespace
	$text =~ s/^\s+//;
	$text =~ s/\s+$//;
	return '' unless $text;
	$args->{'ts'} = $text;
	require MT::Template::ContextHandlers;
	return MT::Template::Context::_hdlr_date(@_);
}

sub build_value {
# convert and build MT template tags within a passed value.
	my ($ctx, $value) = @_;
	return '' unless $value;
		# within a value argument, you can use MT tags, but with
		# square brackets instead of angle brackets and single quotes
		# instead of double quotes; literal square brackets and single
		# quotes must be escaped with a backslash
		# convert non-escaped []'
	$value =~ s/(?<!\\)\[/</g;
	$value =~ s/(?<!\\)\]/>/g;
	$value =~ s/(?<!\\)'/"/g;
		# de-escape escaped []'
	$value =~ s/\\([\[\]'])/$1/g;
		# any MT tags?
	if ($value =~ /<MT/) {
		my $builder = $ctx->stash('builder');
		my $tok = $builder->compile($ctx, $value);
		$value = $builder->build($ctx, $tok);
		return $ctx->error($builder->errstr) unless defined($value);
	}
	return $value;
}

sub current_ts {
	my ($blog_id) = @_;
	my @ts = offset_time_list(time, $blog_id);
	my $ts = sprintf "%04d%02d%02d%02d%02d%02d",
		$ts[5]+1900, $ts[4]+1, @ts[3,2,1,0];
	return $ts;
}

1;
